@@ -298,9 +298,10 @@ static void do_sys_vm86(struct kernel_vm86_struct *info, struct task_struct *tsk
 	__asm__ __volatile__(
 		"xorl %%eax,%%eax; movl %%eax,%%fs; movl %%eax,%%gs\n\t"
 		"movl %0,%%esp\n\t"
+		"movl %1,%%ebp\n\t"
 		"jmp resume_userspace"
 		: /* no outputs */
-		:"r" (&info->regs), "b" (tsk->thread_info) : "ax");
+		:"r" (&info->regs), "r" (tsk->thread_info) : "ax");
 	/* we never return here */
 }
 
@@ -311,8 +312,9 @@ static inline void return_to_32bit(struct kernel_vm86_regs * regs16, int retval)
 	regs32 = save_v86_state(regs16);
 	regs32->eax = retval;
 	__asm__ __volatile__("movl %0,%%esp\n\t"
+		"movl %1,%%ebp\n\t"
 		"jmp resume_userspace"
-		: : "r" (regs32), "b" (current_thread_info()));
+		: : "r" (regs32), "r" (current_thread_info()));
 }
 
 static inline void set_IF(struct kernel_vm86_regs * regs)
