@@ -61,7 +61,7 @@ unsigned long    cache_decay_ticks = 0;
 extern char vmhalt_cmd[];
 extern char vmpoff_cmd[];
 
-extern void reipl(unsigned long devno);
+extern void do_reipl(unsigned long devno);
 
 static sigp_ccode smp_ext_bitcall(int, ec_bit_sig);
 static void smp_ext_bitcall_others(ec_bit_sig);
@@ -217,7 +217,10 @@ static void do_machine_restart(void * __unused)
 		 * interrupted by an external interrupt and s390irq
 		 * locks are always held disabled).
 		 */
-		reipl(S390_lowcore.ipl_device);
+		if (MACHINE_IS_VM)
+			cpcmd ("IPL", NULL, 0);
+		else
+			do_reipl (0x10000 | S390_lowcore.ipl_device);
 	}
 	signal_processor(smp_processor_id(), sigp_stop);
 }
@@ -274,7 +277,7 @@ void machine_power_off_smp(void)
 
 void do_ext_call_interrupt(struct pt_regs *regs, __u16 code)
 {
-        int bits;
+        unsigned long bits;
 
         /*
          * handle bit signal external calls
@@ -282,9 +285,7 @@ void do_ext_call_interrupt(struct pt_regs *regs, __u16 code)
          * For the ec_schedule signal we have to do nothing. All the work
          * is done automatically when we return from the interrupt.
          */
-        do {
-                bits = atomic_read(&S390_lowcore.ext_call_fast);
-        } while (atomic_compare_and_swap(bits,0,&S390_lowcore.ext_call_fast));
+	bits = xchg(&S390_lowcore.ext_call_fast, 0);
 
 	if (test_bit(ec_call_function, &bits)) 
 		do_call_function();
@@ -296,13 +297,12 @@ void do_ext_call_interrupt(struct pt_regs *regs, __u16 code)
  */
 static sigp_ccode smp_ext_bitcall(int cpu, ec_bit_sig sig)
 {
-        struct _lowcore *lowcore = lowcore_ptr[cpu];
         sigp_ccode ccode;
 
         /*
          * Set signaling bit in lowcore of target cpu and kick it
          */
-        atomic_set_mask(1<<sig, &lowcore->ext_call_fast);
+	set_bit(sig, (unsigned long *) &lowcore_ptr[cpu]->ext_call_fast);
         ccode = signal_processor(cpu, sigp_external_call);
         return ccode;
 }
@@ -323,7 +323,7 @@ static void smp_ext_bitcall_others(ec_bit_sig sig)
                 /*
                  * Set signaling bit in lowcore of target cpu and kick it
                  */
-                atomic_set_mask(1<<sig, &lowcore->ext_call_fast);
+		set_bit(sig, (unsigned long *) &lowcore_ptr[i]->ext_call_fast);
                 while (signal_processor(i, sigp_external_call) == sigp_busy)
 			udelay(10);
         }
@@ -480,7 +480,7 @@ static struct task_struct *__devinit fork_by_hand(void)
        /* don't care about the psw and regs settings since we'll never
           reschedule the forked task. */
        memset(&regs,0,sizeof(struct pt_regs));
-       return do_fork(CLONE_VM|CLONE_IDLETASK, 0, &regs, 0, NULL);
+       return do_fork(CLONE_VM|CLONE_IDLETASK, 0, &regs, 0, NULL, NULL);
 }
 
 int __cpu_up(unsigned int cpu)
@@ -572,6 +572,12 @@ void __init smp_prepare_cpus(unsigned int max_cpus)
 	set_prefix((u32) lowcore_ptr[smp_processor_id()]);
 }
 
+void __devinit smp_prepare_boot_cpu(void)
+{
+	set_bit(smp_processor_id(), &cpu_online_map);
+	set_bit(smp_processor_id(), &cpu_possible_map);
+}
+
 void smp_cpus_done(unsigned int max_cpus)
 {
 }
