@@ -1,12 +1,11 @@
 /******************************************************************************
  *
  * Module Name: dswload - Dispatcher namespace load callbacks
- *              $Revision: 78 $
  *
  *****************************************************************************/
 
 /*
- *  Copyright (C) 2000 - 2002, R. Byron Moore
+ *  Copyright (C) 2000 - 2003, R. Byron Moore
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -39,10 +38,10 @@
 
 /*******************************************************************************
  *
- * FUNCTION:    Acpi_ds_init_callbacks
+ * FUNCTION:    acpi_ds_init_callbacks
  *
- * PARAMETERS:  Walk_state      - Current state of the parse tree walk
- *              Pass_number     - 1, 2, or 3
+ * PARAMETERS:  walk_state      - Current state of the parse tree walk
+ *              pass_number     - 1, 2, or 3
  *
  * RETURN:      Status
  *
@@ -52,8 +51,8 @@
 
 acpi_status
 acpi_ds_init_callbacks (
-	acpi_walk_state         *walk_state,
-	u32                     pass_number)
+	struct acpi_walk_state          *walk_state,
+	u32                             pass_number)
 {
 
 	switch (pass_number) {
@@ -87,9 +86,9 @@ acpi_ds_init_callbacks (
 
 /*******************************************************************************
  *
- * FUNCTION:    Acpi_ds_load1_begin_op
+ * FUNCTION:    acpi_ds_load1_begin_op
  *
- * PARAMETERS:  Walk_state      - Current state of the parse tree walk
+ * PARAMETERS:  walk_state      - Current state of the parse tree walk
  *              Op              - Op that has been just been reached in the
  *                                walk;  Arguments have not been evaluated yet.
  *
@@ -101,18 +100,18 @@ acpi_ds_init_callbacks (
 
 acpi_status
 acpi_ds_load1_begin_op (
-	acpi_walk_state         *walk_state,
-	acpi_parse_object       **out_op)
+	struct acpi_walk_state          *walk_state,
+	union acpi_parse_object         **out_op)
 {
-	acpi_parse_object       *op;
-	acpi_namespace_node     *node;
-	acpi_status             status;
-	acpi_object_type        object_type;
-	NATIVE_CHAR             *path;
-	u32                     flags;
+	union acpi_parse_object         *op;
+	struct acpi_namespace_node      *node;
+	acpi_status                     status;
+	acpi_object_type                object_type;
+	char                            *path;
+	u32                             flags;
 
 
-	ACPI_FUNCTION_NAME ("Ds_load1_begin_op");
+	ACPI_FUNCTION_NAME ("ds_load1_begin_op");
 
 
 	op = walk_state->op;
@@ -226,8 +225,8 @@ acpi_ds_load1_begin_op (
 		 * already exists:
 		 *    1) the Scope() operator can reopen a scoping object that was
 		 *       previously defined (Scope, Method, Device, etc.)
-		 *    2) Whenever we are parsing a deferred opcode (Op_region, Buffer,
-		 *       Buffer_field, or Package), the name of the object is already
+		 *    2) Whenever we are parsing a deferred opcode (op_region, Buffer,
+		 *       buffer_field, or Package), the name of the object is already
 		 *       in the namespace.
 		 */
 		flags = ACPI_NS_NO_UPSEARCH;
@@ -289,9 +288,9 @@ acpi_ds_load1_begin_op (
 
 /*******************************************************************************
  *
- * FUNCTION:    Acpi_ds_load1_end_op
+ * FUNCTION:    acpi_ds_load1_end_op
  *
- * PARAMETERS:  Walk_state      - Current state of the parse tree walk
+ * PARAMETERS:  walk_state      - Current state of the parse tree walk
  *              Op              - Op that has been just been completed in the
  *                                walk;  Arguments have now been evaluated.
  *
@@ -304,14 +303,14 @@ acpi_ds_load1_begin_op (
 
 acpi_status
 acpi_ds_load1_end_op (
-	acpi_walk_state         *walk_state)
+	struct acpi_walk_state          *walk_state)
 {
-	acpi_parse_object       *op;
-	acpi_object_type        object_type;
-	acpi_status             status = AE_OK;
+	union acpi_parse_object         *op;
+	acpi_object_type                object_type;
+	acpi_status                     status = AE_OK;
 
 
-	ACPI_FUNCTION_NAME ("Ds_load1_end_op");
+	ACPI_FUNCTION_NAME ("ds_load1_end_op");
 
 
 	op = walk_state->op;
@@ -340,7 +339,7 @@ acpi_ds_load1_end_op (
 
 	if (op->common.aml_opcode == AML_REGION_OP) {
 		status = acpi_ex_create_region (op->named.data, op->named.length,
-				   (ACPI_ADR_SPACE_TYPE) ((op->common.value.arg)->common.value.integer), walk_state);
+				   (acpi_adr_space_type) ((op->common.value.arg)->common.value.integer), walk_state);
 		if (ACPI_FAILURE (status)) {
 			return (status);
 		}
@@ -356,6 +355,37 @@ acpi_ds_load1_end_op (
 		}
 	}
 
+	if (op->common.aml_opcode == AML_METHOD_OP) {
+		/*
+		 * method_op pkg_length name_string method_flags term_list
+		 *
+		 * Note: We must create the method node/object pair as soon as we
+		 * see the method declaration.  This allows later pass1 parsing
+		 * of invocations of the method (need to know the number of
+		 * arguments.)
+		 */
+		ACPI_DEBUG_PRINT ((ACPI_DB_DISPATCH,
+			"LOADING-Method: State=%p Op=%p named_obj=%p\n",
+			walk_state, op, op->named.node));
+
+		if (!acpi_ns_get_attached_object (op->named.node)) {
+			walk_state->operands[0] = (void *) op->named.node;
+			walk_state->num_operands = 1;
+
+			status = acpi_ds_create_operands (walk_state, op->common.value.arg);
+			if (ACPI_SUCCESS (status)) {
+				status = acpi_ex_create_method (op->named.data,
+						   op->named.length, walk_state);
+			}
+			walk_state->operands[0] = NULL;
+			walk_state->num_operands = 0;
+
+			if (ACPI_FAILURE (status)) {
+				return (status);
+			}
+		}
+	}
+
 	/* Pop the scope stack */
 
 	if (acpi_ns_opens_scope (object_type)) {
@@ -371,9 +401,9 @@ acpi_ds_load1_end_op (
 
 /*******************************************************************************
  *
- * FUNCTION:    Acpi_ds_load2_begin_op
+ * FUNCTION:    acpi_ds_load2_begin_op
  *
- * PARAMETERS:  Walk_state      - Current state of the parse tree walk
+ * PARAMETERS:  walk_state      - Current state of the parse tree walk
  *              Op              - Op that has been just been reached in the
  *                                walk;  Arguments have not been evaluated yet.
  *
@@ -385,17 +415,17 @@ acpi_ds_load1_end_op (
 
 acpi_status
 acpi_ds_load2_begin_op (
-	acpi_walk_state         *walk_state,
-	acpi_parse_object       **out_op)
+	struct acpi_walk_state          *walk_state,
+	union acpi_parse_object         **out_op)
 {
-	acpi_parse_object       *op;
-	acpi_namespace_node     *node;
-	acpi_status             status;
-	acpi_object_type        object_type;
-	NATIVE_CHAR             *buffer_ptr;
+	union acpi_parse_object         *op;
+	struct acpi_namespace_node      *node;
+	acpi_status                     status;
+	acpi_object_type                object_type;
+	char                            *buffer_ptr;
 
 
-	ACPI_FUNCTION_TRACE ("Ds_load2_begin_op");
+	ACPI_FUNCTION_TRACE ("ds_load2_begin_op");
 
 
 	op = walk_state->op;
@@ -425,7 +455,7 @@ acpi_ds_load2_begin_op (
 		else {
 			/* Get name from the op */
 
-			buffer_ptr = (NATIVE_CHAR *) &op->named.name;
+			buffer_ptr = (char *) &op->named.name;
 		}
 	}
 	else {
@@ -454,7 +484,7 @@ acpi_ds_load2_begin_op (
 	case AML_INT_NAMEPATH_OP:
 
 		/*
-		 * The Name_path is an object reference to an existing object. Don't enter the
+		 * The name_path is an object reference to an existing object. Don't enter the
 		 * name into the namespace, but look it up for use later
 		 */
 		status = acpi_ns_lookup (walk_state->scope_info, buffer_ptr, object_type,
@@ -582,9 +612,9 @@ acpi_ds_load2_begin_op (
 
 /*******************************************************************************
  *
- * FUNCTION:    Acpi_ds_load2_end_op
+ * FUNCTION:    acpi_ds_load2_end_op
  *
- * PARAMETERS:  Walk_state      - Current state of the parse tree walk
+ * PARAMETERS:  walk_state      - Current state of the parse tree walk
  *              Op              - Op that has been just been completed in the
  *                                walk;  Arguments have now been evaluated.
  *
@@ -597,20 +627,20 @@ acpi_ds_load2_begin_op (
 
 acpi_status
 acpi_ds_load2_end_op (
-	acpi_walk_state         *walk_state)
+	struct acpi_walk_state          *walk_state)
 {
-	acpi_parse_object       *op;
-	acpi_status             status = AE_OK;
-	acpi_object_type        object_type;
-	acpi_namespace_node     *node;
-	acpi_parse_object       *arg;
-	acpi_namespace_node     *new_node;
+	union acpi_parse_object         *op;
+	acpi_status                     status = AE_OK;
+	acpi_object_type                object_type;
+	struct acpi_namespace_node      *node;
+	union acpi_parse_object         *arg;
+	struct acpi_namespace_node      *new_node;
 #ifndef ACPI_NO_METHOD_EXECUTION
-	u32                     i;
+	u32                             i;
 #endif
 
 
-	ACPI_FUNCTION_TRACE ("Ds_load2_end_op");
+	ACPI_FUNCTION_TRACE ("ds_load2_end_op");
 
 	op = walk_state->op;
 	ACPI_DEBUG_PRINT ((ACPI_DB_DISPATCH, "Opcode [%s] Op %p State %p\n",
@@ -651,11 +681,10 @@ acpi_ds_load2_end_op (
 
 		status = acpi_ds_scope_stack_pop (walk_state);
 		if (ACPI_FAILURE (status)) {
-			return_ACPI_STATUS (status);
+			goto cleanup;
 		}
 	}
 
-
 	/*
 	 * Named operations are as follows:
 	 *
@@ -685,7 +714,7 @@ acpi_ds_load2_end_op (
 	 */
 
 	ACPI_DEBUG_PRINT ((ACPI_DB_DISPATCH,
-		"Create-Load [%s] State=%p Op=%p Named_obj=%p\n",
+		"Create-Load [%s] State=%p Op=%p named_obj=%p\n",
 		acpi_ps_get_opcode_name (op->common.aml_opcode), walk_state, op, node));
 
 	/* Decode the opcode */
@@ -789,30 +818,10 @@ acpi_ds_load2_end_op (
 	case AML_TYPE_NAMED_COMPLEX:
 
 		switch (op->common.aml_opcode) {
-		case AML_METHOD_OP:
-			/*
-			 * Method_op Pkg_length Name_string Method_flags Term_list
-			 */
-			ACPI_DEBUG_PRINT ((ACPI_DB_DISPATCH,
-				"LOADING-Method: State=%p Op=%p Named_obj=%p\n",
-				walk_state, op, node));
-
-			if (!acpi_ns_get_attached_object (node)) {
-				status = acpi_ds_create_operands (walk_state, arg);
-				if (ACPI_FAILURE (status)) {
-					goto cleanup;
-				}
-
-				status = acpi_ex_create_method (op->named.data,
-						   op->named.length, walk_state);
-			}
-			break;
-
-
 #ifndef ACPI_NO_METHOD_EXECUTION
 		case AML_REGION_OP:
 			/*
-			 * The Op_region is not fully parsed at this time. Only valid argument is the Space_id.
+			 * The op_region is not fully parsed at this time. Only valid argument is the space_id.
 			 * (We must save the address of the AML of the address and length operands)
 			 */
 			/*
@@ -842,6 +851,7 @@ acpi_ds_load2_end_op (
 
 		default:
 			/* All NAMED_COMPLEX opcodes must be handled above */
+			/* Note: Method objects were already created in Pass 1 */
 			break;
 		}
 		break;
@@ -856,7 +866,7 @@ acpi_ds_load2_end_op (
 	case AML_CLASS_METHOD_CALL:
 
 		ACPI_DEBUG_PRINT ((ACPI_DB_DISPATCH,
-			"RESOLVING-Method_call: State=%p Op=%p Named_obj=%p\n",
+			"RESOLVING-method_call: State=%p Op=%p named_obj=%p\n",
 			walk_state, op, node));
 
 		/*
@@ -891,7 +901,6 @@ acpi_ds_load2_end_op (
 		break;
 	}
 
-
 cleanup:
 
 	/* Remove the Node pushed at the very beginning */
