@@ -1010,11 +1010,11 @@ static int
 fst_set_iface ( struct fst_card_info *card, struct fst_port_info *port,
                 struct ifreq *ifr )
 {
-	union line_settings *line = &ifr->ifr_settings->ifs_line;
         sync_serial_settings sync;
         int i;
 
-        if ( copy_from_user ( &sync, &line->sync, sizeof ( sync )))
+        if (copy_from_user (&sync, ifr->ifr_settings.ifs_ifsu.sync,
+			    sizeof (sync)))
                 return -EFAULT;
 
         if ( sync.loopback )
@@ -1022,7 +1022,7 @@ fst_set_iface ( struct fst_card_info *card, struct fst_port_info *port,
 
         i = port->index;
 
-        switch ( ifr->ifr_settings->type )
+        switch (ifr->ifr_settings.type)
         {
         case IF_IFACE_V35:
                 FST_WRW ( card, portConfig[i].lineInterface, V35 );
@@ -1067,7 +1067,6 @@ static int
 fst_get_iface ( struct fst_card_info *card, struct fst_port_info *port,
                 struct ifreq *ifr )
 {
-	union line_settings *line = &ifr->ifr_settings->ifs_line;
         sync_serial_settings sync;
         int i;
 
@@ -1078,24 +1077,30 @@ fst_get_iface ( struct fst_card_info *card, struct fst_port_info *port,
         switch ( port->hwif )
         {
         case V35:
-                ifr->ifr_settings->type = IF_IFACE_V35;
+                ifr->ifr_settings.type = IF_IFACE_V35;
                 break;
         case V24:
-                ifr->ifr_settings->type = IF_IFACE_V24;
+                ifr->ifr_settings.type = IF_IFACE_V24;
                 break;
         case X21:
         default:
-                ifr->ifr_settings->type = IF_IFACE_X21;
+                ifr->ifr_settings.type = IF_IFACE_X21;
                 break;
         }
 
+	if (ifr->ifr_settings.size < sizeof(sync)) {
+		ifr->ifr_settings.size = sizeof(sync); /* data size wanted */
+		return -ENOBUFS;
+	}
+
         i = port->index;
         sync.clock_rate = FST_RDL ( card, portConfig[i].lineSpeed );
         /* Lucky card and linux use same encoding here */
         sync.clock_type = FST_RDB ( card, portConfig[i].internalClock );
         sync.loopback = 0;
 
-        if ( copy_to_user (&line->sync, &sync, sizeof ( sync )))
+        if (copy_to_user (ifr->ifr_settings.ifs_ifsu.sync, &sync,
+			  sizeof(sync)))
                 return -EFAULT;
 
         return 0;
@@ -1221,7 +1226,7 @@ fst_ioctl ( struct net_device *dev, struct ifreq *ifr, int cmd )
                 return set_conf_from_info ( card, port, &info );
 
         case SIOCWANDEV:
-                switch ( ifr->ifr_settings->type )
+                switch (ifr->ifr_settings.type)
                 {
                 case IF_GET_IFACE:
                         return fst_get_iface ( card, port, ifr );
@@ -1660,12 +1665,12 @@ fst_remove_one ( struct pci_dev *pdev )
 }
 
 static struct pci_driver fst_driver = {
-        name:           FST_NAME,
-        id_table:       fst_pci_dev_id,
-        probe:          fst_add_one,
-        remove:         __devexit_p(fst_remove_one),
-        suspend:        NULL,
-        resume:         NULL,
+        .name		= FST_NAME,
+        .id_table	= fst_pci_dev_id,
+        .probe		= fst_add_one,
+        .remove	= __devexit_p(fst_remove_one),
+        .suspend	= NULL,
+        .resume	= NULL,
 };
 
 static int __init
