@@ -57,6 +57,12 @@ static inline char *portspeed (int portstatus)
 }
 #endif
 
+/* for dev_info, dev_dbg, etc */
+static inline struct device *hubdev (struct usb_device *dev)
+{
+	return &dev->actconfig->interface [0].dev;
+}
+
 /* USB 2.0 spec Section 11.24.4.5 */
 static int usb_get_hub_descriptor(struct usb_device *dev, void *data, int size)
 {
@@ -257,7 +263,8 @@ static void usb_hub_power_on(struct usb_hub *hub)
 	int i;
 
 	/* Enable power to the ports */
-	dbg("enabling power on all ports");
+	dev_dbg(hubdev(interface_to_usbdev(hub->intf)),
+		"enabling power on all ports\n");
 	dev = interface_to_usbdev(hub->intf);
 	for (i = 0; i < hub->descriptor->bNbrPorts; i++)
 		usb_set_port_feature(dev, i + 1, USB_PORT_FEAT_POWER);
@@ -270,15 +277,17 @@ static int usb_hub_configure(struct usb_hub *hub,
 	struct usb_endpoint_descriptor *endpoint)
 {
 	struct usb_device *dev = interface_to_usbdev (hub->intf);
+	struct device *hub_dev;
 	struct usb_hub_status hubstatus;
 	unsigned int pipe;
 	int maxp, ret;
+	char *message;
 
 	hub->descriptor = kmalloc(sizeof(*hub->descriptor), GFP_KERNEL);
 	if (!hub->descriptor) {
-		err("Unable to kmalloc %Zd bytes for hub descriptor",
-			sizeof(*hub->descriptor));
-		return -1;
+		message = "can't kmalloc hub descriptor";
+		ret = -ENOMEM;
+		goto fail;
 	}
 
 	/* Request the entire hub descriptor.
@@ -288,17 +297,17 @@ static int usb_hub_configure(struct usb_hub *hub,
 	ret = usb_get_hub_descriptor(dev, hub->descriptor,
 			sizeof(*hub->descriptor));
 	if (ret < 0) {
-		err("Unable to get hub descriptor (err = %d)", ret);
-		kfree(hub->descriptor);
-		return -1;
+		message = "can't read hub descriptor";
+		goto fail;
 	} else if (hub->descriptor->bNbrPorts > USB_MAXCHILDREN) {
-		err("Hub is too big! %d children", hub->descriptor->bNbrPorts);
-		kfree(hub->descriptor);
-		return -1;
+		message = "hub has too many ports!";
+		ret = -ENODEV;
+		goto fail;
 	}
 
+	hub_dev = hubdev(dev);
 	dev->maxchild = hub->descriptor->bNbrPorts;
-	info("%d port%s detected", dev->maxchild,
+	dev_info (hub_dev, "%d port%s detected\n", dev->maxchild,
 		(dev->maxchild == 1) ? "" : "s");
 
 	le16_to_cpus(&hub->descriptor->wHubCharacteristics);
@@ -312,33 +321,33 @@ static int usb_hub_configure(struct usb_hub *hub,
 				    [((i + 1) / 8)] & (1 << ((i + 1) % 8))
 				? 'F' : 'R';
 		portstr[dev->maxchild] = 0;
-		dbg("compound device; port removable status: %s", portstr);
+		dev_dbg(hub_dev, "compound device; port removable status: %s\n", portstr);
 	} else
-		dbg("standalone hub");
+		dev_dbg(hub_dev, "standalone hub\n");
 
 	switch (hub->descriptor->wHubCharacteristics & HUB_CHAR_LPSM) {
 		case 0x00:
-			dbg("ganged power switching");
+			dev_dbg(hub_dev, "ganged power switching\n");
 			break;
 		case 0x01:
-			dbg("individual port power switching");
+			dev_dbg(hub_dev, "individual port power switching\n");
 			break;
 		case 0x02:
 		case 0x03:
-			dbg("unknown reserved power switching mode");
+			dev_dbg(hub_dev, "unknown reserved power switching mode\n");
 			break;
 	}
 
 	switch (hub->descriptor->wHubCharacteristics & HUB_CHAR_OCPM) {
 		case 0x00:
-			dbg("global over-current protection");
+			dev_dbg(hub_dev, "global over-current protection\n");
 			break;
 		case 0x08:
-			dbg("individual port over-current protection");
+			dev_dbg(hub_dev, "individual port over-current protection\n");
 			break;
 		case 0x10:
 		case 0x18:
-			dbg("no over-current protection");
+			dev_dbg(hub_dev, "no over-current protection\n");
                         break;
 	}
 
@@ -349,16 +358,16 @@ static int usb_hub_configure(struct usb_hub *hub,
 		case 0:
 			break;
 		case 1:
-			dbg("Single TT");
+			dev_dbg(hub_dev, "Single TT\n");
 			hub->tt.hub = dev;
 			break;
 		case 2:
-			dbg("TT per port");
+			dev_dbg(hub_dev, "TT per port\n");
 			hub->tt.hub = dev;
 			hub->tt.multi = 1;
 			break;
 		default:
-			dbg("Unrecognized hub protocol %d",
+			dev_dbg(hub_dev, "Unrecognized hub protocol %d\n",
 				dev->descriptor.bDeviceProtocol);
 			break;
 	}
@@ -366,42 +375,41 @@ static int usb_hub_configure(struct usb_hub *hub,
 	switch (hub->descriptor->wHubCharacteristics & HUB_CHAR_TTTT) {
 		case 0x00:
 			if (dev->descriptor.bDeviceProtocol != 0)
-				dbg("TT requires at most 8 FS bit times");
+				dev_dbg(hub_dev, "TT requires at most 8 FS bit times\n");
 			break;
 		case 0x20:
-			dbg("TT requires at most 16 FS bit times");
+			dev_dbg(hub_dev, "TT requires at most 16 FS bit times\n");
 			break;
 		case 0x40:
-			dbg("TT requires at most 24 FS bit times");
+			dev_dbg(hub_dev, "TT requires at most 24 FS bit times\n");
 			break;
 		case 0x60:
-			dbg("TT requires at most 32 FS bit times");
+			dev_dbg(hub_dev, "TT requires at most 32 FS bit times\n");
 			break;
 	}
 
-	dbg("Port indicators are %s supported", 
+	dev_dbg(hub_dev, "Port indicators are %s supported\n", 
 	    (hub->descriptor->wHubCharacteristics & HUB_CHAR_PORTIND)
 	    	? "" : "not");
 
-	dbg("power on to power good time: %dms",
+	dev_dbg(hub_dev, "power on to power good time: %dms\n",
 		hub->descriptor->bPwrOn2PwrGood * 2);
-	dbg("hub controller current requirement: %dmA",
+	dev_dbg(hub_dev, "hub controller current requirement: %dmA\n",
 		hub->descriptor->bHubContrCurrent);
 
 	ret = usb_get_hub_status(dev, &hubstatus);
 	if (ret < 0) {
-		err("Unable to get hub status (err = %d)", ret);
-		kfree(hub->descriptor);
-		return -1;
+		message = "can't get hub status";
+		goto fail;
 	}
 
 	le16_to_cpus(&hubstatus.wHubStatus);
 
-	dbg("local power source is %s",
+	dev_dbg(hub_dev, "local power source is %s\n",
 		(hubstatus.wHubStatus & HUB_STATUS_LOCAL_POWER)
 		? "lost (inactive)" : "good");
 
-	dbg("%sover-current condition exists",
+	dev_dbg(hub_dev, "%sover-current condition exists\n",
 		(hubstatus.wHubStatus & HUB_STATUS_OVERCURRENT) ? "" : "no ");
 
 	/* Start the interrupt endpoint */
@@ -413,18 +421,17 @@ static int usb_hub_configure(struct usb_hub *hub,
 
 	hub->urb = usb_alloc_urb(0, GFP_KERNEL);
 	if (!hub->urb) {
-		err("couldn't allocate interrupt urb");
-		kfree(hub->descriptor);
-		return -1;
+		message = "couldn't allocate interrupt urb";
+		ret = -ENOMEM;
+		goto fail;
 	}
 
 	usb_fill_int_urb(hub->urb, dev, pipe, hub->buffer, maxp, hub_irq,
 		hub, endpoint->bInterval);
 	ret = usb_submit_urb(hub->urb, GFP_KERNEL);
 	if (ret) {
-		err("usb_submit_urb failed (%d)", ret);
-		kfree(hub->descriptor);
-		return -1;
+		message = "couldn't submit status urb";
+		goto fail;
 	}
 		
 	/* Wake up khubd */
@@ -433,17 +440,23 @@ static int usb_hub_configure(struct usb_hub *hub,
 	usb_hub_power_on(hub);
 
 	return 0;
+
+fail:
+	dev_err (&hub->intf->dev, "config failed, %s (err %d)\n",
+			message, ret);
+	/* hub_disconnect() frees urb and descriptor */
+	return ret;
 }
 
 static void hub_disconnect(struct usb_interface *intf)
 {
-	struct usb_hub *hub = dev_get_drvdata (&intf->dev);
+	struct usb_hub *hub = usb_get_intfdata (intf);
 	unsigned long flags;
 
 	if (!hub)
 		return;
 
-	dev_set_drvdata (&intf->dev, NULL);
+	usb_set_intfdata (intf, NULL);
 	spin_lock_irqsave(&hub_event_lock, flags);
 
 	/* Delete it and then reset it */
@@ -491,37 +504,32 @@ static int hub_probe(struct usb_interface *intf, const struct usb_device_id *id)
 	/*  specs is not defined, but it works */
 	if ((desc->desc.bInterfaceSubClass != 0) &&
 	    (desc->desc.bInterfaceSubClass != 1)) {
-		err("invalid subclass (%d) for USB hub device #%d",
-			desc->desc.bInterfaceSubClass, dev->devnum);
+descriptor_error:
+		dev_err (&intf->dev, "bad descriptor, ignoring hub\n");
 		return -EIO;
 	}
 
 	/* Multiple endpoints? What kind of mutant ninja-hub is this? */
 	if (desc->desc.bNumEndpoints != 1) {
-		err("invalid bNumEndpoints (%d) for USB hub device #%d",
-			desc->desc.bNumEndpoints, dev->devnum);
-		return -EIO;
+		goto descriptor_error;
 	}
 
 	endpoint = &desc->endpoint[0].desc;
 
 	/* Output endpoint? Curiousier and curiousier.. */
 	if (!(endpoint->bEndpointAddress & USB_DIR_IN)) {
-		err("Device #%d is hub class, but has output endpoint?",
-			dev->devnum);
-		return -EIO;
+		goto descriptor_error;
 	}
 
 	/* If it's not an interrupt endpoint, we'd better punt! */
 	if ((endpoint->bmAttributes & USB_ENDPOINT_XFERTYPE_MASK)
 			!= USB_ENDPOINT_XFER_INT) {
-		err("Device #%d is hub class, but endpoint is not interrupt?",
-			dev->devnum);
+		goto descriptor_error;
 		return -EIO;
 	}
 
 	/* We found a hub */
-	info("USB hub found at %s", dev->devpath);
+	dev_info (hubdev (dev), "USB hub found\n");
 
 	hub = kmalloc(sizeof(*hub), GFP_KERNEL);
 	if (!hub) {
@@ -541,15 +549,13 @@ static int hub_probe(struct usb_interface *intf, const struct usb_device_id *id)
 	list_add(&hub->hub_list, &hub_list);
 	spin_unlock_irqrestore(&hub_event_lock, flags);
 
-	dev_set_drvdata (&intf->dev, hub);
+	usb_set_intfdata (intf, hub);
 
 	if (usb_hub_configure(hub, endpoint) >= 0) {
 		strcpy (intf->dev.name, "Hub");
 		return 0;
 	}
 
-	err("hub configuration failed for device at %s", dev->devpath);
-
 	hub_disconnect (intf);
 	return -ENODEV;
 }
@@ -651,8 +657,6 @@ static int usb_hub_port_status(struct usb_device *hub, int port,
 		else {
 			*status = le16_to_cpu(portsts->wPortStatus);
 			*change = le16_to_cpu(portsts->wPortChange); 
-			dbg("port %d, portstatus %x, change %x, %s", port + 1,
-				*status, *change, portspeed(*status));
 			ret = 0;
 		}
 		kfree(portsts);
@@ -710,8 +714,9 @@ static int usb_hub_port_wait_reset(struct usb_device *hub, int port,
 		if (delay_time >= 2 * HUB_SHORT_RESET_TIME)
 			delay = HUB_LONG_RESET_TIME;
 
-		dbg("port %d of hub %s not reset yet, waiting %dms", port + 1,
-			hub->devpath, delay);
+		dev_dbg (hubdev (hub),
+			"port %d not reset yet, waiting %dms\n",
+			port + 1, delay);
 	}
 
 	return -1;
@@ -735,14 +740,15 @@ static int usb_hub_port_reset(struct usb_device *hub, int port,
 			return status;
 		}
 
-		dbg("port %d of hub %s not enabled, trying reset again...",
-			port + 1, hub->devpath);
+		dev_dbg (hubdev (hub),
+			"port %d not enabled, trying reset again...\n",
+			port + 1);
 		delay = HUB_LONG_RESET_TIME;
 	}
 
-	err("Cannot enable port %i of hub %s, disabling port.",
-		port + 1, hub->devpath);
-	err("Maybe the USB cable is bad?");
+	dev_err (hubdev (hub),
+		"Cannot enable port %i.  Maybe the USB cable is bad?\n",
+		port + 1);
 
 	return -1;
 }
@@ -768,35 +774,50 @@ void usb_hub_port_disable(struct usb_device *hub, int port)
  * Not covered by the spec - but easy to deal with.
  *
  * This implementation uses 400ms minimum debounce timeout and checks
- * every 100ms for transient disconnects to restart the delay.
+ * every 25ms for transient disconnects to restart the delay.
  */
 
 #define HUB_DEBOUNCE_TIMEOUT	400
-#define HUB_DEBOUNCE_STEP	100
+#define HUB_DEBOUNCE_STEP	 25
+#define HUB_DEBOUNCE_STABLE	  4
 
 /* return: -1 on error, 0 on success, 1 on disconnect.  */
 static int usb_hub_port_debounce(struct usb_device *hub, int port)
 {
 	int ret;
-	unsigned delay_time;
+	int delay_time, stable_count;
 	u16 portchange, portstatus;
+	unsigned connection;
 
-	for (delay_time = 0; delay_time < HUB_DEBOUNCE_TIMEOUT; /* empty */ ) {
-
-		/* wait debounce step increment */
+	connection = 0;
+	stable_count = 0;
+	for (delay_time = 0; delay_time < HUB_DEBOUNCE_TIMEOUT; delay_time += HUB_DEBOUNCE_STEP) {
 		wait_ms(HUB_DEBOUNCE_STEP);
 
 		ret = usb_hub_port_status(hub, port, &portstatus, &portchange);
 		if (ret < 0)
 			return -1;
 
+		if ((portstatus & USB_PORT_STAT_CONNECTION) == connection) {
+			if (connection) {
+				if (++stable_count == HUB_DEBOUNCE_STABLE)
+					break;
+			}
+		} else {
+			stable_count = 0;
+		}
+		connection = portstatus & USB_PORT_STAT_CONNECTION;
+
 		if ((portchange & USB_PORT_STAT_C_CONNECTION)) {
 			usb_clear_port_feature(hub, port+1, USB_PORT_FEAT_C_CONNECTION);
-			delay_time = 0;
 		}
-		else
-			delay_time += HUB_DEBOUNCE_STEP;
 	}
+
+	/* XXX Replace this with dbg() when 2.6 is about to ship. */
+	dev_info (hubdev (hub),
+		"debounce: port %d: delay %dms stable %d status 0x%x\n",
+		port + 1, delay_time, stable_count, portstatus);
+
 	return ((portstatus&USB_PORT_STAT_CONNECTION)) ? 0 : 1;
 }
 
@@ -808,9 +829,9 @@ static void usb_hub_port_connect_change(struct usb_hub *hubstate, int port,
 	unsigned int delay = HUB_SHORT_RESET_TIME;
 	int i;
 
-	dbg("hub %s port %d, portstatus %x, change %x, %s",
-		hub->devpath, port + 1,
-		portstatus, portchange, portspeed (portstatus));
+	dev_dbg (&hubstate->intf->dev,
+		"port %d, status %x, change %x, %s\n",
+		port + 1, portstatus, portchange, portspeed (portstatus));
 
 	/* Clear the connection change status */
 	usb_clear_port_feature(hub, port + 1, USB_PORT_FEAT_C_CONNECTION);
@@ -828,7 +849,9 @@ static void usb_hub_port_connect_change(struct usb_hub *hubstate, int port,
 	}
 
 	if (usb_hub_port_debounce(hub, port)) {
-		err("connect-debounce failed, port %d disabled", port+1);
+		dev_err (&hubstate->intf->dev,
+			"connect-debounce failed, port %d disabled\n",
+			port+1);
 		usb_hub_port_disable(hub, port);
 		return;
 	}
@@ -847,7 +870,8 @@ static void usb_hub_port_connect_change(struct usb_hub *hubstate, int port,
 		/* Allocate a new device struct */
 		dev = usb_alloc_dev(hub, hub->bus);
 		if (!dev) {
-			err("couldn't allocate usb_device");
+			dev_err (&hubstate->intf->dev,
+				"couldn't allocate usb_device\n");
 			break;
 		}
 
@@ -855,7 +879,7 @@ static void usb_hub_port_connect_change(struct usb_hub *hubstate, int port,
 
 		/* Reset the device, and detect its speed */
 		if (usb_hub_port_reset(hub, port, dev, delay)) {
-			usb_free_dev(dev);
+			usb_put_dev(dev);
 			break;
 		}
 
@@ -890,10 +914,12 @@ static void usb_hub_port_connect_change(struct usb_hub *hubstate, int port,
 			len = snprintf (dev->devpath, sizeof dev->devpath,
 				"%d", port + 1);
 		if (len == sizeof dev->devpath)
-			warn ("devpath size! usb/%03d/%03d path %s",
+			dev_err (&hubstate->intf->dev,
+				"devpath size! usb/%03d/%03d path %s\n",
 				dev->bus->busnum, dev->devnum, dev->devpath);
-		info("new USB device %s-%s, assigned address %d",
-			dev->bus->bus_name, dev->devpath, dev->devnum);
+		dev_info (&hubstate->intf->dev,
+			"new USB device on port %d, assigned address %d\n",
+			port + 1, dev->devnum);
 
 		/* put the device in the global device tree. the hub port
 		 * is the "bus_id"; hubs show in hierarchy like bridges
@@ -905,7 +931,7 @@ static void usb_hub_port_connect_change(struct usb_hub *hubstate, int port,
 			goto done;
 
 		/* Free the configuration if there was an error */
-		usb_free_dev(dev);
+		usb_put_dev(dev);
 
 		/* Switch to a long reset time */
 		delay = HUB_LONG_RESET_TIME;
@@ -978,12 +1004,11 @@ static void usb_hub_events(void)
 			}
 
 			if (portchange & USB_PORT_STAT_C_CONNECTION) {
-				dbg("hub %s port %d connection change",
-					dev->devpath, i + 1);
 				usb_hub_port_connect_change(hub, i, portstatus, portchange);
 			} else if (portchange & USB_PORT_STAT_C_ENABLE) {
-				dbg("hub %s port %d enable change, status %x",
-					dev->devpath, i + 1, portstatus);
+				dev_dbg (hubdev (dev),
+					"port %d enable change, status %x\n",
+					i + 1, portstatus);
 				usb_clear_port_feature(dev,
 					i + 1, USB_PORT_FEAT_C_ENABLE);
 
